from dataclasses import dataclass
from datetime import datetime
from typing import Optional

from delphinium.entities.base import HeliotropeEntity


@dataclass
class Info(HeliotropeEntity):
    id: int
    title: str
    thumbnail: str
    artists: list[str]
    groups: list[str]
    type: str
    language: Optional[str]
    series: list[str]
    characters: list[str]
    tags: list[str]
    date: datetime



from delphinium.entities import *
from delphinium.http import PhloxHTTP


class Delphinium(PhloxHTTP):
    async def galleryinfo(self, index: int) -> Galleryinfo:
        response = await self.get_galleryinfo(index)
        return Galleryinfo.from_dict(response)

    async def image(self, index: int) -> list[str]:
        resp = await self.get_image(index)
        return resp["files"]

    async def search(self, query: list[str], offset: int = 1) -> tuple[list[Info], int]:
        resp = await self.post_search(query, offset)
        return (
            [Info.from_dict(info) for info in resp["result"]],
            resp["count"],
        )

    async def random(self, query: list[str]) -> Info:
        resp = await self.get_random(query)
        return Info.from_dict(resp)

    async def info(self, index: int) -> Info:
        response = await self.get_info(index)
        return Info.from_dict(response)

    async def list(self, index: int) -> tuple[list[Info], int]:
        resp = await self.get_list(index)
        infos = [Info.from_dict(info) for info in resp["list"]]
        return infos, resp["total"]



from typing import Optional, TypedDict


class BaseHeliotropeJSON(TypedDict): ...


class HeliotropeFileJSON(BaseHeliotropeJSON):
    hasavif: bool
    hash: str
    height: int
    name: str
    width: int
    hasjxl: bool
    haswebp: bool
    single: bool


class HeliotropeTagJSON(BaseHeliotropeJSON):
    tag: str
    url: str
    female: bool
    male: bool


class HeliotropeParodysJSON(BaseHeliotropeJSON):
    parody: str
    url: str


class HeliotropeArtistsJSON(BaseHeliotropeJSON):
    artist: str
    url: str


class HeliotropeCharactersJSON(BaseHeliotropeJSON):
    character: str
    url: str


class HeliotropeGroupsJSON(BaseHeliotropeJSON):
    group: str
    url: str


class HeliotropeLanguagesJSON(BaseHeliotropeJSON):
    galleryid: str
    language_localname: str
    name: str
    url: str


class HeliotropeGalleryinfoJSON(BaseHeliotropeJSON):
    galleryurl: str
    id: int
    japanese_title: Optional[str]
    language_localname: str
    language_url: str
    language: str
    title: str
    type: str
    video: Optional[str]
    videofilename: Optional[str]
    blocked: bool
    datepublished: Optional[str]
    artists: list[HeliotropeArtistsJSON]
    characters: list[HeliotropeCharactersJSON]
    files: list[HeliotropeFileJSON]
    groups: list[HeliotropeGroupsJSON]
    languages: list[HeliotropeLanguagesJSON]
    parodys: list[HeliotropeParodysJSON]
    related: list[int]
    scene_indexes: list[int]
    tags: list[HeliotropeTagJSON]
    date: str


class HeliotropeInfoJSON(BaseHeliotropeJSON):
    id: int
    title: str
    thumbnail: str
    artist: list[str]
    group: list[str]
    type: str
    language: Optional[str]
    series: list[str]
    character: list[str]
    tag: list[str]
    date: str


class HeliotropeFilesJSON(BaseHeliotropeJSON):
    files: list[str]


class HeliotropeListJSON(BaseHeliotropeJSON):
    list: list[HeliotropeInfoJSON]
    total: int


class HeliotropeSearchJSON(BaseHeliotropeJSON):
    result: list[HeliotropeInfoJSON]
    count: int



from platform import python_version
from types import TracebackType
from typing import Any, Literal, Optional

from aiohttp import ClientSession
from aiohttp import __version__ as aiohttp_version

from delphinium import __version__ as delphinium_version
from delphinium.error import PhloxHTTPError
from delphinium.types import (
    HeliotropeFilesJSON,
    HeliotropeGalleryinfoJSON,
    HeliotropeInfoJSON,
    HeliotropeListJSON,
    HeliotropeSearchJSON,
)


class PhloxHTTP:
    UA = f"Delphinium (https://github.com/Saebasol/Delphinium {delphinium_version}) Python/{python_version()} aiohttp/{aiohttp_version}"

    def __init__(
        self, base_url: str, client_session: Optional[ClientSession] = None
    ) -> None:
        self.base_url = base_url.rstrip("/")
        self.client_session = client_session

    async def request(
        self,
        method: Literal["GET", "POST"],
        path: str,
        json: Optional[dict[str, Any]] = None,
    ) -> Any:
        url = self.base_url + path

        if not self.client_session:
            self.client_session = ClientSession()

        async with self.client_session.request(
            method, url, json=json, headers={"user-agent": self.UA}
        ) as resp:
            body = await resp.json()

            if resp.status != 200:
                raise PhloxHTTPError(body["message"])

            return body

    async def get_galleryinfo(self, index: int) -> HeliotropeGalleryinfoJSON:
        return await self.request("GET", f"/api/hitomi/galleryinfo/{index}")

    async def get_image(self, index: int) -> HeliotropeFilesJSON:
        return await self.request("GET", f"/api/hitomi/image/{index}")

    async def get_info(self, index: int) -> HeliotropeInfoJSON:
        return await self.request("GET", f"/api/hitomi/info/{index}")

    async def get_list(self, index: int) -> HeliotropeListJSON:
        return await self.request("GET", f"/api/hitomi/list/{index}")

    async def get_random(self, query: list[str]) -> HeliotropeInfoJSON:
        return await self.request("POST", "/api/hitomi/random", {"query": query})

    async def post_search(self, query: list[str], offset: int) -> HeliotropeSearchJSON:
        return await self.request(
            "POST",
            "/api/hitomi/search",
            {
                "query": query,
                "offset": offset,
            },
        )

    async def __aenter__(self) -> "PhloxHTTP":
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        if self.client_session:
            await self.client_session.close()



from dataclasses import dataclass, field

from delphinium.entities.base import HeliotropeEntity


@dataclass
class File(HeliotropeEntity):
    hasavif: bool
    hash: str
    height: int
    name: str
    width: int
    hasjxl: bool = field(default=False)
    haswebp: bool = field(default=False)
    single: bool = field(default=False)
